// Code adapted from: http://en.highscore.de/cpp/boost/parser.html


// Spirit Qi is the parser library of Spirit which
// enables one to generate parsers starting from EBNF
// specifications
// For more information, you can look at:
// http://www.boost.org/doc/libs/1_56_0/libs/spirit/doc/html/spirit/introduction.html
#include <boost/spirit/include/qi.hpp>

using namespace boost::spirit;

// create the CFG that will correspond to our JSON parser
// REMARKS:
// Clearly, we cannot use the EBNF syntax as is inside C++,
// so Boost.Spirit employs a few modifications in order to
// make it conforming to the C++ syntax rules
// Specifically, we do not have a 'empty' operator so the
// right shift operator '>>' is used instead
// (e.g. 'a >> b' instead of 'a b')
// The assignment operator '=' is used in place of the EBNF
// '::=' operator. The or ('|') operator and the paranthesis
// ('(' and ')') remain unchanged inside the corresponding
// Also, the Kleene star * is a postfix operator inside EBNF,
// but it is overloaded as a prefix * inside Spirit
// (e.g. a* becomes *a)

template <typename T>
        struct json : qi::grammar<T>
        {
            // non-terminals of the grammar
            qi::rule<T> object, member, str, value, number, array;
            // the starting rule: a whole JSON object
            json() : json::base_type(object)
            {
                // the production rules of the grammar
                object = "{" >> member >> *("," >> member) >> "}";
                member = str >> ":" >> value;
                str = "\"" >> *~qi::lit('"') >> "\"";
                value = str | number | object | array | "true" | "false" | "null";
                number = qi::double_;
                array = "[" >> value >> *("," >> value) >> "]";
            }
        };

int main()
{
    // A JSON object to test the parser
    std::string s = "{\"Boost Spirit Parser\":{\"Language\":\"C++\",\"Non "
            "Terminals\":[\"object\",\"member\",\"str\",\"value\",\"number\",\"array\"]}}";
    // construct the parser
    json<std::string::iterator> json_parser;
    // run the parser to see if the given string is a valid JSON object
    bool success = qi::parse(s.begin(), s.end(), json_parser);
    std::cout << (success ? "OK" : "Error") << std::endl;

}
